@@ -1,5 +1,7 @@
 import os
+import copy
 
+import numpy as np
 import pandas as pd
 
 from itertools import islice
@@ -18,9 +20,9 @@ from boris.api._utils import check_image
 from boris.api._utils import PIL_to_bytes
 
 from boris.data import BorisDataset
+from boris.utils import fit_pca
 from boris.utils import pandas_to_dict
 
-
 import tqdm
 import warnings
 from concurrent.futures import ThreadPoolExecutor
@@ -56,9 +58,26 @@ def upload_embeddings_from_csv(path_to_embeddings: str,
         msg = 'Forbidden upload to dataset with no existing tags.'
         raise RuntimeError(msg)
 
+    # use pca to make 2d embeddings
+    transformer = fit_pca(df.values[:, 1:-1])
+
+    def _make_2d_embedding(batch):
+        batch_2d = copy.deepcopy(batch)
+        for embedding in batch_2d['embeddings']:
+            value_2d = transformer.transform(
+                np.array(embedding['value'])[None, :]
+            ).ravel()
+            embedding['value'] = value_2d.tolist()
+        return batch_2d
+
     def _upload_single_batch(batch, append=False):
         batch['append'] = int(append)
+        batch['type'] = 'model'
+        batch_2d = _make_2d_embedding(batch)
+        batch_2d['embeddingName'] = batch_2d['embeddingName'] + '__2D'
+        batch_2d['type'] = 'pca'
         upload_embedding(batch)
+        upload_embedding(batch_2d)
 
     n_embeddings = len(data['embeddings'])
     n_batches = n_embeddings // max_upload
@@ -145,7 +164,8 @@ def upload_images_from_folder(path_to_folder: str,
                     thumbname, dataset_id, sample_id, token
                 )
                 # try to create thumbnail
-                with Image.open(os.path.join(path_to_folder, fname)) as temp_image:
+                image_path = os.path.join(path_to_folder, fname)
+                with Image.open(image_path) as temp_image:
                     thumbnail = get_thumbnail_from_img(temp_image)
                 # try to upload thumbnail
                 upload_file_with_signed_url(
@@ -164,7 +184,8 @@ def upload_images_from_folder(path_to_folder: str,
                     basename, dataset_id, sample_id, token
                 )
                 # try to upload image
-                with open(os.path.join(path_to_folder, fname), 'rb') as temp_image:
+                image_path = os.path.join(path_to_folder, fname)
+                with open(image_path, 'rb') as temp_image:
                     upload_file_with_signed_url(
                         temp_image,
                         signed_url
