@@ -83,7 +83,8 @@ def _download_cli(cfg, is_cli_call=True):
 
         print(f'Copying files from {from_folder} to {to_folder}.')
         for i in tqdm(indices):
-            os.makedirs(target_names[i], exist_ok=True)
+            dirname = os.path.dirname(target_names[i])
+            os.makedirs(dirname, exist_ok=True)
             shutil.copy(source_names[i], target_names[i])
 
 
